import { interfaces } from '@theia/core/shared/inversify';
import { PreferenceProxy, PreferenceService, PreferenceSchema } from '@theia/core/lib/browser';
export declare const TerminalConfigSchema: PreferenceSchema;
export declare type Profiles = null | {
    [key: string]: {
        path?: string | string[];
        source?: string;
        args?: string | string[];
        env?: {
            [key: string]: string;
        };
        overrideName?: boolean;
        icon?: string;
        color?: string;
    };
};
export interface TerminalConfiguration {
    'terminal.enableCopy': boolean;
    'terminal.enablePaste': boolean;
    'terminal.integrated.fontFamily': string;
    'terminal.integrated.fontSize': number;
    'terminal.integrated.fontWeight': FontWeight;
    'terminal.integrated.fontWeightBold': FontWeight;
    'terminal.integrated.drawBoldTextInBrightColors': boolean;
    'terminal.integrated.letterSpacing': number;
    'terminal.integrated.lineHeight': number;
    'terminal.integrated.scrollback': number;
    'terminal.integrated.fastScrollSensitivity': number;
    'terminal.integrated.rendererType': TerminalRendererType;
    'terminal.integrated.copyOnSelection': boolean;
    'terminal.integrated.cursorBlinking': boolean;
    'terminal.integrated.cursorStyle': CursorStyleVSCode;
    'terminal.integrated.cursorWidth': number;
    'terminal.integrated.shell.windows': string | null | undefined;
    'terminal.integrated.shell.osx': string | null | undefined;
    'terminal.integrated.shell.linux': string | null | undefined;
    'terminal.integrated.shellArgs.windows': string[];
    'terminal.integrated.shellArgs.osx': string[];
    'terminal.integrated.shellArgs.linux': string[];
    'terminal.integrated.defaultProfile.windows': string;
    'terminal.integrated.defaultProfile.linux': string;
    'terminal.integrated.defaultProfile.osx': string;
    'terminal.integrated.profiles.windows': Profiles;
    'terminal.integrated.profiles.linux': Profiles;
    'terminal.integrated.profiles.osx': Profiles;
    'terminal.integrated.confirmOnExit': ConfirmOnExitType;
    'terminal.integrated.enablePersistentSessions': boolean;
}
declare type FontWeight = 'normal' | 'bold' | '100' | '200' | '300' | '400' | '500' | '600' | '700' | '800' | '900';
export declare type CursorStyle = 'block' | 'underline' | 'bar';
export declare type CursorStyleVSCode = CursorStyle | 'line';
export declare type TerminalRendererType = 'canvas' | 'dom';
export declare type ConfirmOnExitType = 'never' | 'always' | 'hasChildProcesses';
export declare const DEFAULT_TERMINAL_RENDERER_TYPE = "canvas";
export declare function isTerminalRendererType(arg: unknown): arg is TerminalRendererType;
export declare const TerminalPreferenceContribution: unique symbol;
export declare const TerminalPreferences: unique symbol;
export declare type TerminalPreferences = PreferenceProxy<TerminalConfiguration>;
export declare function createTerminalPreferences(preferences: PreferenceService, schema?: PreferenceSchema): TerminalPreferences;
export declare function bindTerminalPreferences(bind: interfaces.Bind): void;
export {};
//# sourceMappingURL=terminal-preferences.d.ts.map